@@ -562,8 +562,9 @@ describe("filter", function()
             {code = "111", name = "not_print", indexing = {"not_print"}, line = 1, column = 1},
             {code = "111", name = "print", indexing = {"print"}, line = 5, column = 1},
             {code = "121", name = "print", indexing = {"print"}, line = 7, column = 1},
-            {code = "021", msg = "invalid value of inline option 'std'", line = 8, column = 1},
-            {code = "021", msg = "invalid value of inline option 'std'", line = 1000, column = 20}
+            {code = "021", msg = "invalid value of option 'std': unknown std 'bad_std'", line = 8, column = 1},
+            {code = "021", msg = "invalid value of option 'std': unknown std 'another_bad_std'",
+               line = 1000,column = 20}
          }
       }, filter_full({
          {
