@@ -1,12 +1,16 @@
 local lexer = require "luacheck.lexer"
 local utils = require "luacheck.utils"
 
+local parser = {}
+
 local function new_state(src)
    return {
       lexer = lexer.new_state(src),
       code_lines = {}, -- Set of line numbers containing code.
+      line_endings = {}, -- Maps line numbers to "comment", "string", or nil based on whether
+                         -- the line ending is within a token.
       comments = {}, -- Array of {comment = string, location = location}.
-      hanging_semicolons = {} -- Array of locations of semicolons not following an expression or goto.
+      hanging_semicolons = {} -- Array of locations of semicolons not following a statement.
    }
 end
 
@@ -18,25 +22,58 @@ local function location(state)
    }
 end
 
+parser.SyntaxError = utils.class()
+
+function parser.SyntaxError:__init(loc, end_column, msg, prev_loc, prev_end_column)
+   self.line = loc.line
+   self.column = loc.column
+   self.end_column = end_column
+   self.msg = msg
+
+   if prev_loc then
+      self.prev_line = prev_loc.line
+      self.prev_column = prev_loc.column
+      self.prev_end_column = prev_end_column
+   end
+end
+
+function parser.syntax_error(loc, end_column, msg, prev_loc, prev_end_column)
+   error(parser.SyntaxError(loc, end_column, msg, prev_loc, prev_end_column), 0)
+end
+
 local function token_body_or_line(state)
    return state.lexer.src:sub(state.offset, state.lexer.offset - 1):match("^[^\r\n]*")
 end
 
+local function mark_line_endings(state, first_line, last_line, token_type)
+   for line = first_line, last_line - 1 do
+      state.line_endings[line] = token_type
+   end
+end
+
 local function skip_token(state)
    while true do
       local err_end_column
-      state.token, state.token_value, state.line, state.column, state.offset, err_end_column = lexer.next_token(state.lexer)
+      state.token, state.token_value, state.line,
+         state.column, state.offset, err_end_column = lexer.next_token(state.lexer)
 
       if not state.token then
-         lexer.syntax_error(state, err_end_column, state.token_value)
+         parser.syntax_error(state, err_end_column, state.token_value)
       elseif state.token == "comment" then
          state.comments[#state.comments+1] = {
             contents = state.token_value,
             location = location(state),
             end_column = state.column + #token_body_or_line(state) - 1
          }
+
+         mark_line_endings(state, state.line, state.lexer.line, "comment")
       else
-         state.code_lines[state.line] = true
+         if state.token ~= "eof" then
+            mark_line_endings(state, state.line, state.lexer.line, "string")
+            state.code_lines[state.line] = true
+            state.code_lines[state.lexer.line] = true
+         end
+
          break
       end
    end
@@ -67,7 +104,7 @@ local function token_name(token)
    return token_names[token] or lexer.quote(token)
 end
 
-local function parse_error(state, msg)
+local function parse_error(state, msg, prev_loc, prev_end_column)
    local token_repr, end_column
 
    if state.token == "eof" then
@@ -79,7 +116,7 @@ local function parse_error(state, msg)
       token_repr = lexer.quote(token_repr)
    end
 
-   lexer.syntax_error(state, end_column, msg .. " near " .. token_repr)
+   parser.syntax_error(state, end_column, msg .. " near " .. token_repr, prev_loc, prev_end_column)
 end
 
 local function check_token(state, token)
@@ -100,15 +137,15 @@ local function test_and_skip_token(state, token)
    end
 end
 
-local function check_closing_token(state, opening_token, closing_token, opening_line)
+local function check_closing_token(state, opening_token, closing_token, opening_loc)
    if state.token ~= closing_token then
       local err = "expected " .. token_name(closing_token)
 
-      if opening_line ~= state.line then
-         err = err .. " (to close " .. token_name(opening_token) .. " on line " .. tostring(opening_line) .. ")"
+      if opening_loc.line ~= state.line then
+         err = err .. " (to close " .. token_name(opening_token) .. " on line " .. tostring(opening_loc.line) .. ")"
       end
 
-      parse_error(state, err)
+      parse_error(state, err, opening_loc, opening_loc.column + #opening_token - 1)
    end
 
    skip_token(state)
@@ -171,7 +208,7 @@ simple_expressions["..."] = atom("Dots")
 
 simple_expressions["{"] = function(state)
    local ast_node = new_ast_node(state, "Table")
-   local start_line = state.line
+   local start_location = location(state)
    skip_token(state)
    local is_inside_parentheses = false
 
@@ -207,7 +244,7 @@ simple_expressions["{"] = function(state)
             first_key_token = "["
             skip_token(state)
             lhs = parse_expression(state)
-            check_closing_token(state, "[", "]", item_location.line)
+            check_closing_token(state, "[", "]", item_location)
             check_and_skip_token(state, "=")
             rhs = parse_expression(state)
          else
@@ -225,14 +262,14 @@ simple_expressions["{"] = function(state)
       end
    until not (test_and_skip_token(state, ",") or test_and_skip_token(state, ";"))
 
-   check_closing_token(state, "{", "}", start_line)
+   check_closing_token(state, "{", "}", start_location)
    opt_add_parens(ast_node, is_inside_parentheses)
    return ast_node
 end
 
 -- Parses argument list and the statements.
 local function parse_function(state, func_location)
-   local paren_line = state.line
+   local paren_location = location(state)
    check_and_skip_token(state, "(")
    local args = {}
 
@@ -249,10 +286,10 @@ local function parse_function(state, func_location)
       until not test_and_skip_token(state, ",")
    end
 
-   check_closing_token(state, "(", ")", paren_line)
+   check_closing_token(state, "(", ")", paren_location)
    local body = parse_block(state)
    local end_location = location(state)
-   check_closing_token(state, "function", "end", func_location.line)
+   check_closing_token(state, "function", "end", func_location)
    return init_ast_node({args, body, end_location = end_location}, func_location, "Function")
 end
 
@@ -265,10 +302,10 @@ end
 local calls = {}
 
 calls["("] = function(state)
-   local paren_line = state.line
+   local paren_location = location(state)
    skip_token(state) -- Skip "(".
    local args = (state.token == ")") and {} or parse_expression_list(state)
-   check_closing_token(state, "(", ")", paren_line)
+   check_closing_token(state, "(", ")", paren_location)
    return args
 end
 
@@ -287,10 +324,10 @@ suffixes["."] = function(state, lhs)
 end
 
 suffixes["["] = function(state, lhs)
-   local bracket_line = state.line
+   local bracket_location = location(state)
    skip_token(state)  -- Skip "[".
    local rhs = parse_expression(state)
-   check_closing_token(state, "[", "]", bracket_line)
+   check_closing_token(state, "[", "]", bracket_location)
    return init_ast_node({lhs, rhs}, lhs.location, "Index")
 end
 
@@ -319,11 +356,11 @@ local function parse_simple_expression(state, kind, no_literals)
 
    if state.token == "(" then
       in_parens = true
-      local paren_line = state.line
+      local paren_location = location(state)
       skip_token(state)
       local _
       expression, _, first_token = parse_expression(state)
-      check_closing_token(state, "(", ")", paren_line)
+      check_closing_token(state, "(", ")", paren_location)
    elseif state.token == "name" then
       expression = parse_id(state)
       first_token = expression[1]
@@ -446,8 +483,8 @@ end
 local statements = {}
 
 statements["if"] = function(state, loc)
-   local start_line, start_token
-   local next_line, next_token = loc.line, "if"
+   local start_location, start_token
+   local next_location, next_token = loc, "if"
    local ast_node = init_ast_node({}, loc, "If")
 
    repeat
@@ -455,18 +492,18 @@ statements["if"] = function(state, loc)
       local branch_location = location(state)
       check_and_skip_token(state, "then")
       ast_node[#ast_node+1] = parse_block(state, branch_location)
-      start_line, start_token = next_line, next_token
-      next_line, next_token = state.line, state.token
+      start_location, start_token = next_location, next_token
+      next_location, next_token = location(state), state.token
    until not test_and_skip_token(state, "elseif")
 
    if state.token == "else" then
-      start_line, start_token = next_line, next_token
+      start_location, start_token = next_location, next_token
       local branch_location = location(state)
       skip_token(state)
       ast_node[#ast_node+1] = parse_block(state, branch_location)
    end
 
-   check_closing_token(state, start_token, "end", start_line)
+   check_closing_token(state, start_token, "end", start_location)
    return ast_node
 end
 
@@ -474,13 +511,13 @@ statements["while"] = function(state, loc)
    local condition = parse_expression(state, "condition")
    check_and_skip_token(state, "do")
    local block = parse_block(state)
-   check_closing_token(state, "while", "end", loc.line)
+   check_closing_token(state, "while", "end", loc)
    return init_ast_node({condition, block}, loc, "While")
 end
 
 statements["do"] = function(state, loc)
    local ast_node = init_ast_node(parse_block(state), loc, "Do")
-   check_closing_token(state, "do", "end", loc.line)
+   check_closing_token(state, "do", "end", loc)
    return ast_node
 end
 
@@ -521,13 +558,13 @@ statements["for"] = function(state, loc)
       parse_error(state, "expected '=', ',' or 'in'")
    end
 
-   check_closing_token(state, "for", "end", loc.line)
+   check_closing_token(state, "for", "end", loc)
    return ast_node
 end
 
 statements["repeat"] = function(state, loc)
    local block = parse_block(state)
-   check_closing_token(state, "repeat", "until", loc.line)
+   check_closing_token(state, "repeat", "until", loc)
    local condition = parse_expression(state, "condition", true)
    return init_ast_node({block, condition}, loc, "Repeat")
 end
@@ -635,7 +672,7 @@ local function parse_expression_statement(state, loc)
 
       if in_parens then
          -- (expr) is invalid.
-         lexer.syntax_error(first_loc, first_loc.column, "expected " .. expected .. " near '('")
+         parser.syntax_error(first_loc, first_loc.column, "expected " .. expected .. " near '('")
       end
 
       if primary_expression.tag == "Call" or primary_expression.tag == "Invoke" then
@@ -711,14 +748,17 @@ end
 
 -- Parses source string.
 -- Returns AST (in almost MetaLua format), array of comments - tables {comment = string, location = location},
--- set of line numbers containing code, and array of locations of empty statements (semicolons).
--- On error throws {line = line, column = column, end_column = end_column, msg = msg}
-local function parse(src)
+-- set of line numbers containing code, map of types of tokens wrapping line endings (nil, "string", or "comment"),
+-- and array of locations of empty statements (semicolons).
+-- On error throws an instance of parser.SyntaxError: a table with required fields `line`, `column`,
+-- `end_column`, and `msg`, and also optional fields `prev_line`, `prev_column`, and `prev_end_column` if the error
+-- refers to some other location.
+function parser.parse(src)
    local state = new_state(src)
    skip_token(state)
    local ast = parse_block(state)
    check_token(state, "eof")
-   return ast, state.comments, state.code_lines, state.hanging_semicolons
+   return ast, state.comments, state.code_lines, state.line_endings, state.hanging_semicolons
 end
 
-return parse
+return parser
