@@ -1,3 +1,5 @@
+local unpack = table.unpack or unpack -- luacheck: compat
+
 local utils = {}
 
 utils.dir_sep = package.config:sub(1,1)
@@ -118,6 +120,12 @@ function utils.update(t1, t2)
    return t1
 end
 
+function utils.remove(t1, t2)
+   for k in pairs(t2) do
+      t1[k] = nil
+   end
+end
+
 local class_metatable = {}
 
 function class_metatable.__call(class, ...)
@@ -136,6 +144,10 @@ function utils.class()
    return class
 end
 
+function utils.is_instance(object, class)
+   return rawequal(debug.getmetatable(object), class)
+end
+
 utils.Stack = utils.class()
 
 function utils.Stack:__init()
@@ -156,32 +168,38 @@ function utils.Stack:pop()
    return value
 end
 
-local function error_handler(err)
-   return {
-      err = err,
-      traceback = debug.traceback()
-   }
+local ErrorWrapper = utils.class()
+
+function ErrorWrapper:__init(err, traceback)
+   self.err = err
+   self.traceback = traceback
 end
 
--- Calls f with arg, returns what it does.
--- If f throws a table, returns nil, the table.
--- If f throws not a table, rethrows.
-function utils.pcall(f, arg)
-   local function task()
-      return f(arg)
-   end
+function ErrorWrapper:__tostring()
+   return tostring(self.err) .. "\n" .. self.traceback
+end
 
-   local ok, res = xpcall(task, error_handler)
-
-   if ok then
-      return res
-   elseif type(res.err) == "table" then
-      return nil, res.err
+local function error_handler(err)
+   if utils.is_instance(err, ErrorWrapper) then
+      return err
    else
-      error(tostring(res.err) .. "\n" .. res.traceback, 0)
+      return ErrorWrapper(err, debug.traceback())
    end
 end
 
+-- Like pcall, but wraps errors in {err = err, traceback = traceback}
+-- tables unless already wrapped.
+function utils.try(f, ...)
+   local args = {...}
+   local num_args = select("#", ...)
+
+   local function task()
+      return f(unpack(args, 1, num_args))
+   end
+
+   return xpcall(task, error_handler)
+end
+
 local function ripairs_iterator(array, i)
    if i == 1 then
       return nil
@@ -195,6 +213,30 @@ function utils.ripairs(array)
    return ripairs_iterator, array, #array + 1
 end
 
+function utils.sorted_pairs(t)
+   local keys = {}
+
+   for key in pairs(t) do
+      table.insert(keys, key)
+   end
+
+   table.sort(keys)
+
+   local index = 1
+
+   return function()
+      local key = keys[index]
+
+      if key == nil then
+         return
+      end
+
+      index = index + 1
+
+      return key, t[key]
+   end
+end
+
 function utils.unprefix(str, prefix)
    if str:sub(1, #prefix) == prefix then
       return str:sub(#prefix + 1)
@@ -269,9 +311,20 @@ function utils.split_lines(str)
    return lines
 end
 
+utils.InvalidPatternError = utils.class()
+
+function utils.InvalidPatternError:__init(err, pattern)
+   self.err = err
+   self.pattern = pattern
+end
+
+function utils.InvalidPatternError:__tostring()
+   return self.err
+end
+
 -- Behaves like string.match, except it normally returns boolean and
--- throws a table {pattern = pattern} on invalid pattern.
--- The error message turns into original error when tostring is used on it,
+-- throws an instance of utils.InvalidPatternError on invalid pattern.
+-- The error object turns into original error when tostring is used on it,
 -- to ensure behaviour is predictable when luacheck is used as a module.
 function utils.pmatch(str, pattern)
    assert(type(str) == "string")
@@ -280,7 +333,7 @@ function utils.pmatch(str, pattern)
    local ok, res = pcall(string.match, str, pattern)
 
    if not ok then
-      error(setmetatable({pattern = pattern}, {__tostring = function() return res end}))
+      error(utils.InvalidPatternError(res, pattern), 0)
    else
       return not not res
    end
@@ -297,24 +350,55 @@ function utils.map(func, array)
    return res
 end
 
--- Returns predicate checking type.
+-- Returns validator checking type.
 function utils.has_type(type_)
    return function(x)
-      return type(x) == type_
+      if type(x) == type_ then
+         return true
+      else
+         return false, ("%s expected, got %s"):format(type_, type(x))
+      end
    end
 end
 
--- Returns predicate checking that value is an array with
--- elements of type.
+-- Returns validator checking type and allowing false.
+function utils.has_type_or_false(type_)
+   return function(x)
+      if type(x) == type_ then
+         return true
+      elseif type(x) == "boolean" then
+         if x then
+            return false, ("%s or false expected, got true"):format(type_)
+         else
+            return true
+         end
+      else
+         return false, ("%s or false expected, got %s"):format(type_, type(x))
+      end
+   end
+end
+
+-- Returns validator checking two type possibilities.
+function utils.has_either_type(type1, type2)
+   return function(x)
+      if type(x) == type1 or type(x) == type2 then
+         return true
+      else
+         return false, ("%s or %s expected, got %s"):format(type1, type2, type(x))
+      end
+   end
+end
+
+-- Returns validator checking that value is an array with elements of type.
 function utils.array_of(type_)
    return function(x)
       if type(x) ~= "table" then
-         return false
+         return false, ("array of %ss expected, got %s"):format(type_, type(x))
       end
 
-      for _, item in ipairs(x) do
+      for index, item in ipairs(x) do
          if type(item) ~= type_ then
-            return false
+            return false, ("array of %ss expected, got %s at index [%d]"):format(type_, type(item), index)
          end
       end
 
@@ -322,11 +406,4 @@ function utils.array_of(type_)
    end
 end
 
--- Returns predicate chacking if value satisfies on of predicates.
-function utils.either(pred1, pred2)
-   return function(x)
-      return pred1(x) or pred2(x)
-   end
-end
-
 return utils
